import * as fs from 'fs';
import * as path from 'path';
import { $css } from 'taddy';
import { getCacheDir, getRootDir } from './config.js';

let LAST_INDEX = 0;
const STYLES = [];
function getStylesState() {
    const { rules } = $css.ruleInjector.styleSheet;
    const newRules = rules.slice(LAST_INDEX);
    // there is also "appending" rules, that don't change the index, but change the rule
    // TODO: think how to handle that
    LAST_INDEX = rules.length;
    const added = [...newRules].map((rule) => rule.cssText || '');
    STYLES.push(...added);
    return {
        added,
    };
}
function resolveFilepath(getter, ...params) {
    if (typeof getter !== 'string') {
        // @ts-expect-error TODO: fix getter params
        return getter(...params);
    }
    return path.join(getRootDir(), getter);
}
function getMtime(filepath) {
    return Number(fs.statSync?.(filepath).mtime);
}
const filesMap = new Map();
const contentMap = new Map();
function readFileSync(filepath) {
    let data = '';
    try {
        const mtime = getMtime(filepath);
        if (filesMap.has(mtime)) {
            return filesMap.get(mtime);
        }
        data = fs.readFileSync?.(filepath).toString();
        if (!data)
            return '';
        filesMap.set(mtime, data);
        contentMap.set(data, mtime);
    }
    catch (e) {
        console.error('error', e);
    }
    return data;
}
function appendFile(filepath, append) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    // // TODO: think about asynchronous appending
    if ('appendFileSync' in fs) {
        fs.appendFileSync(filepath, append);
    }
    else {
        // workaround for some special FS cases like "filer"
        // @ts-expect-error doesn't exit for some reason
        fs.appendFile(filepath, append, () => void 0);
    }
}
/** don't merge declarations in plugin, currently considering only dev mode */
$css.ruleInjector.styleSheet.options.mergeDeclarations = false;
class Output {
    env;
    config;
    filepath;
    constructor({ env, config }) {
        this.env = env;
        this.config = Object.assign({
            // getCSSFilename = (content) => `atoms-${stringHash(content)}.css`,
            cssFilename: () => `taddy.css`,
            cssFilepath: (filename) => path.join(getCacheDir(), filename),
        }, config);
        const filename = resolveFilepath(this.config.cssFilename);
        this.filepath = resolveFilepath(this.config.cssFilepath, filename);
        // console.log('filepath', this.filepath);
        /* clean cache */
        // fs.writeFileSync(this.filepath, '');
    }
    save /* {sourceMap, filename} = {} */() {
        const stylesData = readFileSync(this.filepath);
        const { added } = getStylesState();
        // TODO: improve filter performance
        const diffStyles = added
            .filter((x) => !stylesData.includes(x))
            .join('');
        // fs.writeFileSync(path.join(getCacheDir(), stringHash(filename) + '.css'), added.join('').replace(/}$/, sourceMap + '}'))
        appendFile(this.filepath, diffStyles);
        // appendFile(this.filepath, sourceMap);
    }
}

export { Output as default };
//# sourceMappingURL=Output.js.map
