import * as t from '@babel/types';
import type { NodePath } from '@babel/traverse';
import { SourceMapGenerator } from 'source-map';
import type { Env } from './types';
import TSProcessor from './TSProcessor';
import type { TSProcessorOptions } from './TSProcessor';
import { PluginPass } from '@babel/core';
type ObjectProperties = t.ObjectExpression['properties'];
export type ProcessorConfig = {
    typescript?: boolean | TSProcessorOptions;
    evaluate?: boolean;
    CSSVariableFallback?: boolean;
    optimizeBindings?: boolean;
};
export type ProcessOptions = {
    env: Env;
    mixin: boolean;
    filename: string;
    code: string;
    addImport(name: string): t.ImportSpecifier['local'];
    sourceMap?: string;
    state?: PluginPass;
    sourceMapGenerator?: SourceMapGenerator;
};
type CommonOptions = {
    postfix?: string;
};
type ObjectOptions = CommonOptions & {
    properties: ObjectProperties;
};
export declare class Processor {
    config: ProcessorConfig;
    options: ProcessOptions;
    isStatic: boolean;
    classNameNodes: WeakSet<t.ObjectProperty>;
    optimizationPaths: Set<NodePath<any>>;
    variables: t.ObjectProperty[];
    private prepare;
    constructor({ config }: {
        config: ProcessorConfig;
    });
    get tsProject(): TSProcessor;
    $css(rule: any, options?: any): import("taddy/$css").InternalTaddyStyle;
    isClassNameNode(node: any): boolean;
    classNamesToNode(classNames: object): t.ObjectProperty[];
    stylesToNode(key: string, value: unknown, { postfix }?: {
        postfix?: string | undefined;
    }): t.ObjectProperty[];
    processPropertyValue(key: string, path: NodePath<t.ObjectProperty>, { postfix, properties }: ObjectOptions): void;
    processObjectProperty(path: NodePath<t.ObjectProperty>, { postfix, properties }: ObjectOptions): void;
    processSpreadElement(path: NodePath<t.SpreadElement>, { postfix, properties }: ObjectOptions): void;
    processLogicalExpression(path: NodePath<t.LogicalExpression>, options: CommonOptions): void;
    processObjectExpression(path: NodePath<t.ObjectExpression>, { postfix }: CommonOptions): void;
    processConditionalExpression(path: NodePath<t.ConditionalExpression>, options?: CommonOptions): void;
    process(path: NodePath<any>, options?: CommonOptions): void;
    run(callPath: NodePath<t.CallExpression>, options: ProcessOptions): {
        isStatic: any;
        optimizationPaths: Set<NodePath<any>>;
    };
}
export {};
