'use strict';

var standalone = require('@babel/standalone');
var register = require('@babel/register');
var evaluatePath = require('babel-helper-evaluate-path');
var resolve = require('resolve');
var path = require('path');
var taddy = require('taddy');
var config = require('../config.cjs');
var findBindings = require('./findBindings.cjs');
var buildCodeByPath = require('./buildCodeByPath.cjs');
var tsPreset = require('@babel/preset-typescript');
var reactPreset = require('@babel/preset-react');
var envPreset = require('@babel/preset-env');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var taddy__namespace = /*#__PURE__*/_interopNamespaceDefault(taddy);

standalone.registerPreset('@babel/preset-typescript', tsPreset);
standalone.registerPreset('@babel/preset-react', reactPreset);
standalone.registerPreset('@babel/preset-env', envPreset);
const DEFAULT_PRESETS = [
    ['@babel/preset-typescript', { allExtensions: true, isTSX: true }],
    '@babel/preset-react',
    [
        '@babel/preset-env',
        {
            targets: { node: '12' },
            useBuiltIns: false,
            ignoreBrowserslistConfig: true,
        },
    ],
];
const EXTENSIONS = ['.es6', '.es', '.tsx', '.ts', '.jsx', '.js', '.mjs'];
const macroRe = new RegExp(config.MACRO_NAME.replace('.', '\\.'), 'g');
const EVAL_FILENAME_POSTFIX = '@__TADDY_EVALUATE__';
// webpack "require" critical dependency issue workaround
const nodeRequire = new Function('require', `return typeof require !== 'undefined'
        ? require
        : (typeof globalThis !== 'undefined' ? globalThis : global).require; `)(module.require);
function isTaddyEvaluation(state) {
    return !!state.filename?.includes(EVAL_FILENAME_POSTFIX);
}
function evaluate(currentPath) {
    const result = evaluatePath(currentPath);
    if (result.confident) {
        findBindings.findBindings(currentPath);
        return { value: result.value };
    }
    let content = '';
    let code = '';
    try {
        const callbackName = '__taddy__';
        content = buildCodeByPath.buildCodeByPath(currentPath)
            // hack to avoid processing by babel-macro
            .replace(macroRe, config.PACKAGE_NAME)
            .concat(`\n\n;${callbackName}(${currentPath.toString()})`);
        // TODO: improve Hub type
        const { opts } = currentPath.hub.file;
        const ext = path.extname(opts.filename);
        const basename = path.basename(opts.filename, ext);
        const dirname = path.dirname(opts.filename);
        const filename = path.join(dirname, basename + EVAL_FILENAME_POSTFIX + ext);
        const options = {
            babelrc: false,
            configFile: false,
            filename,
            plugins: [
            /*...opts.plugins*/
            ],
            presets: [/*...opts.presets*/ ...DEFAULT_PRESETS],
        };
        ({ code } = standalone.transform(content, options) || {});
        if (!code)
            return {};
        const exec = new Function('require', callbackName, code);
        register({
            ...options,
            extensions: EXTENSIONS,
        });
        let value;
        exec((filepath) => {
            if (filepath === config.PACKAGE_NAME)
                return taddy__namespace;
            const requirePath = resolve.sync(filepath, {
                extensions: EXTENSIONS,
                basedir: path.dirname(opts.filename),
            });
            return nodeRequire(requirePath);
        }, (result) => {
            value = result;
        });
        register.revert();
        return { value };
    }
    catch (error) {
        // console.log('evaluate error', {content, code, error});
        return { error };
    }
}

exports.evaluate = evaluate;
exports.isTaddyEvaluation = isTaddyEvaluation;
//# sourceMappingURL=evaluate.cjs.map
