import { registerPreset, transform } from '@babel/standalone';
import register, { revert } from '@babel/register';
import evaluatePath from 'babel-helper-evaluate-path';
import resolve from 'resolve';
import path__default from 'path';
import * as taddy from 'taddy';
import { PACKAGE_NAME, MACRO_NAME } from '../config.js';
import { findBindings } from './findBindings.js';
import { buildCodeByPath } from './buildCodeByPath.js';
import tsPreset from '@babel/preset-typescript';
import reactPreset from '@babel/preset-react';
import envPreset from '@babel/preset-env';

registerPreset('@babel/preset-typescript', tsPreset);
registerPreset('@babel/preset-react', reactPreset);
registerPreset('@babel/preset-env', envPreset);
const DEFAULT_PRESETS = [
    ['@babel/preset-typescript', { allExtensions: true, isTSX: true }],
    '@babel/preset-react',
    [
        '@babel/preset-env',
        {
            targets: { node: '12' },
            useBuiltIns: false,
            ignoreBrowserslistConfig: true,
        },
    ],
];
const EXTENSIONS = ['.es6', '.es', '.tsx', '.ts', '.jsx', '.js', '.mjs'];
const macroRe = new RegExp(MACRO_NAME.replace('.', '\\.'), 'g');
const EVAL_FILENAME_POSTFIX = '@__TADDY_EVALUATE__';
// webpack "require" critical dependency issue workaround
const nodeRequire = new Function('require', `return typeof require !== 'undefined'
        ? require
        : (typeof globalThis !== 'undefined' ? globalThis : global).require; `)(module.require);
function isTaddyEvaluation(state) {
    return !!state.filename?.includes(EVAL_FILENAME_POSTFIX);
}
function evaluate(currentPath) {
    const result = evaluatePath(currentPath);
    if (result.confident) {
        findBindings(currentPath);
        return { value: result.value };
    }
    let content = '';
    let code = '';
    try {
        const callbackName = '__taddy__';
        content = buildCodeByPath(currentPath)
            // hack to avoid processing by babel-macro
            .replace(macroRe, PACKAGE_NAME)
            .concat(`\n\n;${callbackName}(${currentPath.toString()})`);
        // TODO: improve Hub type
        const { opts } = currentPath.hub.file;
        const ext = path__default.extname(opts.filename);
        const basename = path__default.basename(opts.filename, ext);
        const dirname = path__default.dirname(opts.filename);
        const filename = path__default.join(dirname, basename + EVAL_FILENAME_POSTFIX + ext);
        const options = {
            babelrc: false,
            configFile: false,
            filename,
            plugins: [
            /*...opts.plugins*/
            ],
            presets: [/*...opts.presets*/ ...DEFAULT_PRESETS],
        };
        ({ code } = transform(content, options) || {});
        if (!code)
            return {};
        const exec = new Function('require', callbackName, code);
        register({
            ...options,
            extensions: EXTENSIONS,
        });
        let value;
        exec((filepath) => {
            if (filepath === PACKAGE_NAME)
                return taddy;
            const requirePath = resolve.sync(filepath, {
                extensions: EXTENSIONS,
                basedir: path__default.dirname(opts.filename),
            });
            return nodeRequire(requirePath);
        }, (result) => {
            value = result;
        });
        revert();
        return { value };
    }
    catch (error) {
        // console.log('evaluate error', {content, code, error});
        return { error };
    }
}

export { evaluate, isTaddyEvaluation };
//# sourceMappingURL=evaluate.js.map
