'use strict';

var t = require('@babel/types');
var assert = require('assert');
var taddy = require('taddy');
var evaluate = require('./helpers/evaluate.cjs');
require('@taddy/core');
var taggedTemplateToObject = require('./helpers/taggedTemplateToObject.cjs');
var config = require('./config.cjs');
var handlers = require('./handlers.cjs');
var Output = require('./Output.cjs');
var sourceMaps = require('./source-maps.cjs');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

function mapCompileOptions({ evaluate = true, unstable_typescript = false, unstable_CSSVariableFallback = true, unstable_optimizeBindings = true, unstable_useTaggedTemplateLiterals = false, }) {
    return {
        evaluate,
        typescript: unstable_typescript,
        CSSVariableFallback: unstable_CSSVariableFallback,
        optimizeBindings: unstable_optimizeBindings,
        useTaggedTemplateLiterals: unstable_useTaggedTemplateLiterals,
    };
}
let output;
// let sourceMapGenerator;
function macro({ references, babel, state, config: _config = {}, }) {
    const { env = config.getEnv(babel), ...config$1 } = _config;
    if (evaluate.isTaddyEvaluation(state)) {
        return;
    }
    const program = state.file.path;
    const code = state.file.code;
    const { filename } = state;
    if (!filename) {
        // TODO: consider a fallback
        throw new Error('No filename provided');
    }
    let importPath = null;
    const sourceMapGenerator = sourceMaps.makeSourceMapGenerator(state.file);
    // sourceMapGenerator.setSourceContent(filename, code);
    taddy.$css.ruleInjector.reset();
    program.traverse({
        ImportDeclaration(p) {
            const { node } = p;
            if (!(node.source.value === config.PACKAGE_NAME ||
                node.source.value === config.MACRO_NAME)) {
                return;
            }
            importPath = p;
            importPath.node.source = t__namespace.stringLiteral('taddy');
            p.stop();
        },
    });
    const importCache = new Map();
    const { useTaggedTemplateLiterals, ...compileOptions } = mapCompileOptions({
        filename,
        ...config$1.compileOptions,
    });
    const { handlers: handlers$1, finish } = handlers.createHandlers(compileOptions, {
        state,
        sourceMapGenerator,
        env,
        filename,
        code,
        addImport(name) {
            if (!importCache.has(name)) {
                const specifier = t__namespace.importSpecifier(t__namespace.identifier(`_${name}_`), t__namespace.identifier(name));
                importCache.set(name, specifier);
                assert(importPath, 'There is no taddy imports');
                importPath.node.specifiers.push(specifier);
            }
            return importCache.get(name).local;
        },
    });
    for (const key in references) {
        if (!(key in handlers$1))
            continue;
        for (const ref of references[key]) {
            let currentPath = ref.parentPath;
            let currentKey = key;
            if (key === 'css' &&
                currentPath?.isMemberExpression() &&
                'name' in currentPath.node.property &&
                currentPath.node.property.name === 'mixin') {
                currentPath = currentPath.parentPath;
                currentKey = 'mixin';
            }
            if (useTaggedTemplateLiterals &&
                (currentKey === 'css' || currentKey === 'mixin') &&
                currentPath?.isTaggedTemplateExpression()) {
                const obj = taggedTemplateToObject.taggedTemplateToObject(currentPath);
                currentPath.replaceWith(t__namespace.callExpression(currentPath.node.tag, [obj]));
            }
            handlers$1[currentKey](currentPath);
        }
    }
    const { isStatic } = finish();
    if (isStatic && importPath !== null) {
        importPath.node.source =
            t__namespace.stringLiteral('@taddy/core');
    }
    output = output || new Output({ env, config: config$1.outputOptions });
    sourceMaps.convertGeneratorToComment(sourceMapGenerator);
    output.save( /*{sourceMap, filename}*/);
    return {
        keepImports: true,
    };
}

exports.macro = macro;
//# sourceMappingURL=macro-plugin.cjs.map
