import * as t from '@babel/types';
import assert from 'assert';
import { $css } from 'taddy';
import { isTaddyEvaluation } from './helpers/evaluate.js';
import '@taddy/core';
import { taggedTemplateToObject } from './helpers/taggedTemplateToObject.js';
import { getEnv, PACKAGE_NAME, MACRO_NAME } from './config.js';
import { createHandlers } from './handlers.js';
import Output from './Output.js';
import { makeSourceMapGenerator, convertGeneratorToComment } from './source-maps.js';

function mapCompileOptions({ evaluate = true, unstable_typescript = false, unstable_CSSVariableFallback = true, unstable_optimizeBindings = true, unstable_useTaggedTemplateLiterals = false, }) {
    return {
        evaluate,
        typescript: unstable_typescript,
        CSSVariableFallback: unstable_CSSVariableFallback,
        optimizeBindings: unstable_optimizeBindings,
        useTaggedTemplateLiterals: unstable_useTaggedTemplateLiterals,
    };
}
let output;
// let sourceMapGenerator;
function macro({ references, babel, state, config: _config = {}, }) {
    const { env = getEnv(babel), ...config } = _config;
    if (isTaddyEvaluation(state)) {
        return;
    }
    const program = state.file.path;
    const code = state.file.code;
    const { filename } = state;
    if (!filename) {
        // TODO: consider a fallback
        throw new Error('No filename provided');
    }
    let importPath = null;
    const sourceMapGenerator = makeSourceMapGenerator(state.file);
    // sourceMapGenerator.setSourceContent(filename, code);
    $css.ruleInjector.reset();
    program.traverse({
        ImportDeclaration(p) {
            const { node } = p;
            if (!(node.source.value === PACKAGE_NAME ||
                node.source.value === MACRO_NAME)) {
                return;
            }
            importPath = p;
            importPath.node.source = t.stringLiteral('taddy');
            p.stop();
        },
    });
    const importCache = new Map();
    const { useTaggedTemplateLiterals, ...compileOptions } = mapCompileOptions({
        filename,
        ...config.compileOptions,
    });
    const { handlers, finish } = createHandlers(compileOptions, {
        state,
        sourceMapGenerator,
        env,
        filename,
        code,
        addImport(name) {
            if (!importCache.has(name)) {
                const specifier = t.importSpecifier(t.identifier(`_${name}_`), t.identifier(name));
                importCache.set(name, specifier);
                assert(importPath, 'There is no taddy imports');
                importPath.node.specifiers.push(specifier);
            }
            return importCache.get(name).local;
        },
    });
    for (const key in references) {
        if (!(key in handlers))
            continue;
        for (const ref of references[key]) {
            let currentPath = ref.parentPath;
            let currentKey = key;
            if (key === 'css' &&
                currentPath?.isMemberExpression() &&
                'name' in currentPath.node.property &&
                currentPath.node.property.name === 'mixin') {
                currentPath = currentPath.parentPath;
                currentKey = 'mixin';
            }
            if (useTaggedTemplateLiterals &&
                (currentKey === 'css' || currentKey === 'mixin') &&
                currentPath?.isTaggedTemplateExpression()) {
                const obj = taggedTemplateToObject(currentPath);
                currentPath.replaceWith(t.callExpression(currentPath.node.tag, [obj]));
            }
            handlers[currentKey](currentPath);
        }
    }
    const { isStatic } = finish();
    if (isStatic && importPath !== null) {
        importPath.node.source =
            t.stringLiteral('@taddy/core');
    }
    output = output || new Output({ env, config: config.outputOptions });
    convertGeneratorToComment(sourceMapGenerator);
    output.save( /*{sourceMap, filename}*/);
    return {
        keepImports: true,
    };
}

export { macro };
//# sourceMappingURL=macro-plugin.js.map
